package lbi.relational;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Predicate;

public class Test {

	public static void main(String[] args) {
		/* Examples. */
		
		// a sample tuple
		Tuple t0 = new Tuple();
		t0.setInt("age", 22);
		t0.setString("name", "mario");

		// getting values
		int age = t0.getInt("age");
		String name = t0.getString("name");
		System.out.println("The age of " + name + " is " + age);
		
		// another tuple
		Tuple t1 = new Tuple();
		t1.setInt("age", 23);
		t1.setString("name", "anna");

		// a list of tuple
		ArrayList<Tuple> rows = new ArrayList<Tuple>();
		rows.add(t0);
		rows.add(t1);
		rows.add(t1); // t1 added twice
		
		// a table
		Table table = new Table("sample", rows);
		System.out.println(table);
		
		// projecting
		String []attToProject = {"age"};
		Table proj = table.project(attToProject);
		System.out.println(proj);

		// renaming
		Map<String, String> renameAtts = new HashMap<String, String>();
		renameAtts.put("age", "eta");
		Table ren = table.rename(renameAtts);
		System.out.println(ren);

		// distinct
		Table dis = table.distinct();
		System.out.println(dis);
		
		// selection using an object
		Predicate<Tuple> cond1 = new Cond1();
		Table sel1 = table.select(cond1);
		System.out.println(sel1);
		
		// selection using lambda expression
//		Table sel2 = table.select( (t) -> {return t.getInt("age") < 23;} );
		Table sel2 = table.select( t -> t.getInt("age") < 23 );
		System.out.println(sel2);
		
		// sorting
		String []attSort = {"name"};
		Table sort = table.sort(attSort);
		System.out.println(sort);
		
		// group by
		Table group = table.groupby("age", "n");
		System.out.println(group);
		
		// join
		Table join = table.join(group.rename(renameAtts), "age",  "eta");
		System.out.println(join);
		
		// solving the text of exercise
		// sales table 
		Table sales = new Table("Sales");
		Tuple s1 = new Tuple();
		s1.setString("CustomerName", "C1");
		s1.setString("Product", "P1");
		s1.setInt("Amount", 10);
		sales.insert(s1);
		Tuple s2 = new Tuple();
		s2.setString("CustomerName", "C2");
		s2.setString("Product", "P1");
		s2.setInt("Amount", 20);
		sales.insert(s2);
		System.out.println(sales);
		// customer table
		Table customer = new Table("Customer");
		Tuple c1 = new Tuple();
		c1.setString("Name", "C1");
		c1.setString("Country", "Italy");
		c1.setInt("Age", 25);
		customer.insert(c1);
		Tuple c2 = new Tuple();
		c2.setString("Name", "C2");
		c2.setString("Country", "France");
		c2.setInt("Age", 26);
		customer.insert(c2);
		System.out.println(customer);
		
		// SELECT Name FROM Customer WHERE Age > 20 AND Country=Italy ORDER BY Age
		Table result = 
				customer
				.select( t -> t.getInt("Age") > 20 && t.getString("Country").equals("Italy") )
				.sort( new String[] {"Age"} )
				.project( new String[] {"Name"} );
		System.out.println(result);
			
		// SELECT Country, Count(*) AS TotSales FROM Sales JOIN Customer on CustomerName=Name GROUP BY Country
		result = sales.join(customer, "CustomerName", "Name").groupby("Country", "TotSales");
		System.out.println(result);

	}

}
